@@ -18,6 +18,7 @@ import io
 import os
 import pathlib
 import sys
+from copy import deepcopy
 from dataclasses import dataclass
 
 __all__ = [
@@ -145,19 +146,20 @@ class PWManDatabase(CryptSQL):
 
 			self.__initTables()
 
-			c = self.sqlExec("SELECT category FROM pw;")
+			c = self.sqlExec("SELECT DISTINCT category FROM pw ORDER BY category;")
 			categories = c.fetchAll()
-			categories = uniq(c[0] for c in categories)
-			for category in categories:
-				c = self.sqlExec("SELECT title FROM pw WHERE category=?;",
+			for (category, ) in categories:
+				c = self.sqlExec("SELECT title FROM pw WHERE category=? ORDER BY title;",
 						 (category,))
 				titles = c.fetchAll()
-				titles = sorted(t[0] for t in titles)
-				for title in titles:
+				for (title, ) in titles:
 					c = self.sqlExec("SELECT category, title, user, pw, bulk FROM pw "
-							 "WHERE category=? AND title=?;",
+							 "WHERE category=? AND title=? "
+							 "LIMIT 1;",
 							 (category, title))
 					data = c.fetchOne()
+					if not data:
+						continue
 					c = self.sqlExec("INSERT INTO entries(category, title, user, pw) "
 							 "VALUES(?,?,?,?);",
 							 (data[0], data[1], data[2], data[3]))
@@ -168,9 +170,9 @@ class PWManDatabase(CryptSQL):
 								 (entryId, data[4]))
 			c = self.sqlExec("SELECT name, data FROM info;")
 			infos = c.fetchAll()
-			for info in infos:
+			for name, data in infos:
 				c = self.sqlExec("INSERT INTO globalattr(name, data) VALUES(?,?);",
-						 (info[0], info[1]))
+						 (name, data))
 			c = self.sqlExec("DROP TABLE IF EXISTS pw;")
 			c = self.sqlExec("DROP TABLE IF EXISTS info;")
 			self.sqlVacuum()
@@ -218,30 +220,31 @@ class PWManDatabase(CryptSQL):
 		"""Returns True, if a category exists in the database.
 		category: The name string of the category.
 		"""
-		return category in self.getCategoryNames()
+		c = self.sqlExec("SELECT EXISTS(SELECT 1 FROM entries "
+				 "WHERE category=? "
+				 "LIMIT 1);",
+				 (category,))
+		data = c.fetchOne()
+		return data and data[0]
 
 	def getCategoryNames(self):
 		"""Get all category names in the database.
 		Returns a sorted list of strings.
 		"""
-		c = self.sqlExec("SELECT category FROM entries;")
-		categories = c.fetchAll()
-		if not categories:
-			return []
-		return uniq(c[0] for c in categories)
+		c = self.sqlExec("SELECT DISTINCT category FROM entries "
+				 "ORDER BY category;")
+		return [ data[0] for data in c.fetchAll() ]
 
 	def getEntryTitles(self, category):
 		"""Get all titles from one category in the database.
 		category: The category name string.
 		Returns a sorted list of strings.
 		"""
-		c = self.sqlExec("SELECT title FROM entries WHERE category=?;",
+		c = self.sqlExec("SELECT title FROM entries "
+				 "WHERE category=? "
+				 "ORDER BY title;",
 				 (category,))
-		titles = c.fetchAll()
-		if not titles:
-			return []
-		titles = sorted(t[0] for t in titles)
-		return titles
+		return [ data[0] for data in c.fetchAll() ]
 
 	def getEntry(self, category, title):
 		"""Get an entry from the database.
@@ -250,7 +253,8 @@ class PWManDatabase(CryptSQL):
 		Returns a PWManEntry() instance.
 		"""
 		c = self.sqlExec("SELECT id, category, title, user, pw FROM entries "
-				 "WHERE category=? AND title=?;",
+				 "WHERE category=? AND title=? "
+				 "LIMIT 1;",
 				 (category,
 				  title))
 		data = c.fetchOne()
@@ -390,7 +394,13 @@ class PWManDatabase(CryptSQL):
 		category: The name string of the category.
 		title: The title string of the entry.
 		"""
-		return self.getEntry(category, title) is not None
+		c = self.sqlExec("SELECT EXISTS(SELECT 1 FROM entries "
+				 "WHERE category=? AND title=? "
+				 "LIMIT 1);",
+				 (category,
+				  title))
+		data = c.fetchOne()
+		return data and data[0]
 
 	def addEntry(self, entry):
 		"""Create a new entry in the database.
@@ -431,45 +441,107 @@ class PWManDatabase(CryptSQL):
 				  entry.entryId))
 		self.__setDirty()
 
-	def moveEntry(self, entry, newCategory, newTitle):
-		"""Move an existing entry to a new category and/or set a new entry title.
-		entry: The PWManEntry() instance to move.
-		newCategory: The new category name string.
-		newTitle: The new title string.
+	def moveEntry(self, entry, newCategory, newTitle, toDb=None, copy=False):
+		"""Move or copy an existing entry to a new category and/or set a new entry title.
+		entry: The PWManEntry() instance to move/copy.
+		newCategory: The target category name string.
+		newTitle: The target title string.
+		toDb: The target database. Defaults to self.
+		copy: If False, then move. If True, then copy.
 		"""
-		if self.entryExists(newCategory, newTitle):
+		toDb = toDb or self
+		if toDb.entryExists(newCategory, newTitle):
 			raise PWManError("Entry does already exist.")
 		oldEntry = self.getEntry(entry.category, entry.title)
 		if not oldEntry:
 			raise PWManError("Entry does not exist.")
-		entry.category = newCategory
-		entry.title = newTitle
-		c = self.sqlExec("UPDATE entries SET "
-				 "category=?, title=? "
-				 "WHERE id=?;",
-				 (entry.category,
-				  entry.title,
-				  oldEntry.entryId))
-		self.__setDirty()
+		if toDb is self and not copy:
+			entry.category = newCategory
+			entry.title = newTitle
+			c = self.sqlExec("UPDATE entries SET "
+					 "category=?, title=? "
+					 "WHERE id=?;",
+					 (entry.category,
+					  entry.title,
+					  oldEntry.entryId))
+			self.__setDirty()
+		else:
+			newEntry = deepcopy(oldEntry)
+			bulk = self.getEntryBulk(newEntry)
+			attrs = self.getEntryAttrs(newEntry)
+			totp = self.getEntryTotp(newEntry)
+			newEntry.entryId = None
+			newEntry.category = newCategory
+			newEntry.title = newTitle
+			toDb.addEntry(newEntry)
+			if bulk:
+				bulk.bulkId = None
+				toDb.setEntryBulk(bulk)
+			for attr in attrs:
+				attr.attrId = None
+				toDb.setEntryAttr(attr)
+			if totp:
+				totp.totpId = None
+				toDb.setEntryTotp(totp)
+			if not copy:
+				entry.entryId = newEntry.entryId
+				entry.category = newEntry.category
+				entry.title = newEntry.title
+				self.delEntry(oldEntry)
 
-	def moveEntries(self, fromCategory, toCategory):
-		"""Move all entries from one category to another category.
+	def moveEntries(self, fromCategory, toCategory, toDb=None, copy=False):
+		"""Move or copy all entries from one category to another category.
 		fromCategory: The category to move all entries from.
 		toCategory: The (new) category to move all entries to.
+		toDb: The target database. Defaults to self.
+		copy: If False, then move. If True, then copy.
 		"""
+		toDb = toDb or self
 		if not self.categoryExists(fromCategory):
 			raise PWManError("Source category does not exist.")
-		c = self.sqlExec("UPDATE entries SET category=? "
-				 "WHERE category=?;",
-				 (toCategory,
-				  fromCategory))
-		self.__setDirty()
+		if toDb is self and fromCategory == toCategory:
+			return
+		fromTitles = self.getEntryTitles(fromCategory)
+		for fromTitle in fromTitles:
+			if toDb.entryExists(toCategory, fromTitle):
+				raise PWManError("Target entry %s/%s does already exist." % (
+						 toCategory, fromTitle))
+		if toDb is self and not copy:
+			c = self.sqlExec("UPDATE entries SET category=? "
+					 "WHERE category=?;",
+					 (toCategory,
+					  fromCategory))
+			self.__setDirty()
+		else:
+			for fromTitle in fromTitles:
+				entry = self.getEntry(fromCategory, fromTitle)
+				bulk = self.getEntryBulk(entry)
+				attrs = self.getEntryAttrs(entry)
+				totp = self.getEntryTotp(entry)
+				entry.entryId = None
+				entry.category = toCategory
+				toDb.addEntry(entry)
+				if bulk:
+					bulk.bulkId = None
+					toDb.setEntryBulk(bulk)
+				for attr in attrs:
+					attr.attrId = None
+					toDb.setEntryAttr(attr)
+				if totp:
+					totp.totpId = None
+					toDb.setEntryTotp(totp)
+			if not copy:
+				for fromTitle in fromTitles:
+					entry = self.getEntry(fromCategory, fromTitle)
+					self.delEntry(entry)
 
 	def delEntry(self, entry):
 		"""Delete an existing entry from the database.
 		entry: The PWManEntry() instance to delete from the database.
 		"""
-		c = self.sqlExec("SELECT id FROM entries WHERE category=? AND title=?;",
+		c = self.sqlExec("SELECT id FROM entries "
+				 "WHERE category=? AND title=? "
+				 "LIMIT 1;",
 				 (entry.category,
 				  entry.title))
 		entryId = c.fetchOne()
@@ -489,7 +561,8 @@ class PWManDatabase(CryptSQL):
 		c = self.sqlExec("SELECT bulk.id, bulk.data "
 				 "FROM bulk, entries "
 				 "WHERE entries.category=? AND entries.title=? AND "
-				 "bulk.entry = entries.id;",
+				 "bulk.entry = entries.id "
+				 "LIMIT 1;",
 				 (entry.category,
 				  entry.title))
 		data = c.fetchOne()
@@ -508,7 +581,7 @@ class PWManDatabase(CryptSQL):
 		if not entry or entry.entryId is None:
 			raise PWManError("Bulk: Entry does not exist.")
 		if entryBulk.data:
-			c = self.sqlExec("SELECT id FROM bulk WHERE entry=?;",
+			c = self.sqlExec("SELECT id FROM bulk WHERE entry=? LIMIT 1;",
 					 (entry.entryId, ))
 			bulkId = c.fetchOne()
 			if bulkId is None:
@@ -537,7 +610,8 @@ class PWManDatabase(CryptSQL):
 		c = self.sqlExec("SELECT totp.id, totp.key, totp.digits, totp.hash "
 				 "FROM totp, entries "
 				 "WHERE entries.category=? AND entries.title=? AND "
-				 "totp.entry = entries.id;",
+				 "totp.entry = entries.id "
+				 "LIMIT 1;",
 				 (entry.category,
 				  entry.title))
 		data = c.fetchOne()
@@ -558,7 +632,7 @@ class PWManDatabase(CryptSQL):
 		if not entry or entry.entryId is None:
 			raise PWManError("TOTP: Entry does not exist.")
 		if entryTotp.key:
-			c = self.sqlExec("SELECT id FROM totp WHERE entry=?;",
+			c = self.sqlExec("SELECT id FROM totp WHERE entry=? LIMIT 1;",
 					 (entry.entryId, ))
 			totpId = c.fetchOne()
 			if totpId is None:
@@ -592,7 +666,8 @@ class PWManDatabase(CryptSQL):
 		c = self.sqlExec("SELECT entryattr.id, entryattr.name, entryattr.data "
 				 "FROM entryattr, entries "
 				 "WHERE entries.category=? AND entries.title=? AND "
-				 "entryattr.entry = entries.id AND entryattr.name=?;",
+				 "entryattr.entry = entries.id AND entryattr.name=? "
+				 "LIMIT 1;",
 				 (entry.category,
 				  entry.title,
 				  attrName))
@@ -635,7 +710,8 @@ class PWManDatabase(CryptSQL):
 			raise PWManError("Attr: Entry does not exist.")
 		if entryAttr.data:
 			c = self.sqlExec("SELECT id FROM entryattr "
-					 "WHERE entry=? AND name=?;",
+					 "WHERE entry=? AND name=? "
+					 "LIMIT 1;",
 					 (entry.entryId,
 					  entryAttr.name))
 			attrId = c.fetchOne()
@@ -665,7 +741,9 @@ class PWManDatabase(CryptSQL):
 		Returns None, if the attribute does not exist.
 		"""
 		try:
-			c = self.sqlExec("SELECT id, data FROM globalattr WHERE name=?;",
+			c = self.sqlExec("SELECT id, data FROM globalattr "
+					 "WHERE name=? "
+					 "LIMIT 1;",
 					 (name,))
 			data = c.fetchOne()
 			return data[1] if data else None
@@ -679,7 +757,8 @@ class PWManDatabase(CryptSQL):
 		"""
 		if data:
 			c = self.sqlExec("SELECT id FROM globalattr "
-					 "WHERE name=?;",
+					 "WHERE name=? "
+					 "LIMIT 1;",
 					 (name,))
 			attrId = c.fetchOne()
 			if attrId is None:
