@@ -22,6 +22,6 @@ find . \( \
        \) -delete
 
 rm -rf doc/api
-rm -rf build dist .pybuild
+rm -rf build dist .pybuild *.egg-info
 rm -f MANIFEST
 rm -f *.html
