@@ -4,4 +4,5 @@ exec apt install \
 	git \
 	python3 \
 	python3-cffi \
-	python3-pyaes
+	python3-pyaes \
+	python3-pycryptodome
